/*******************************************************************************
 * Copyright (c) 2005, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: ClientHandlerImpl.java,v 1.6 2006-04-06 15:48:06 samwai Exp $
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.internal.collection.framework;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;

import org.eclipse.hyades.execution.security.IClientHandler;

/**
 * @author rduggan
 *
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class ClientHandlerImpl implements IClientHandler {
	
	private Socket _socket;
	
	private InputStream _instrm;
	
	private OutputStream _outstrm;

	public ClientHandlerImpl(Socket s) {
		_socket = s;
		
		try {
			_instrm = s.getInputStream();
		
			_outstrm = s.getOutputStream();
		}
		catch (IOException ioe) {
			_instrm = null;
			_outstrm = null;
		}
	}
	
	public byte[] getRemoteAddress() {
		return _socket.getInetAddress().getAddress();
	}

	public int read(byte[] b, int off, int len) {
		int bytesRead;
		
		try {
			// >>> Bug 128421 begins
			// Workaround to avoid JVM inconsistency of implementation of InputStream.read()
			// In some JSSE implementation the read(buf, off, 0) blocks
			if (len == 0) {
				bytesRead = 0;  // Avoid reading from input stream if there is nothing to read
			}
			else {
				bytesRead = _instrm.read(b, off,len);
			}
			// bytesRead = _instrm.read(b, off,len);
            //	<<< Bug 128421 ends
		}
		catch (IOException ioe) {
			bytesRead = -1;
		}
			
		return bytesRead;
	}
	
	public int write(byte[] b, int off, int len) {
		try {
			_outstrm.write(b, off, len);
		}
		catch (IOException ioe) {
			return -1;
		}
		
		// Multiplexed data channel bug
		return len;
		
	}
	
	public void closeConnection() {
		try {
			_instrm.close();
			_outstrm.close();
		
			_socket.close();
		}
		catch (IOException ioe) {
		}
	}
}