/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: AgentFactory.java,v 1.3 2005-10-01 14:47:02 samwai Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.internal.execution.local.control;

public class AgentFactory {
	
	/**
	 * Create a new agent instance with the specified name.
	 */
	public static Agent createAgent(Process process, String name) {
		return createAgent(process, name, null);
	}
	
	/**
	 * Create a new agent instance with the specified name and type.
	 */
	public static Agent createAgent(Process process, String name, String type) {
		Agent agent = null;

		/*
		 * Try to check if there is an agent already registered with the process
		 */
		agent = process.getAgent(name);
		if(agent != null) { // Assumption: Agent names must be unique within a process 
			// Do nothing since the agent already exist
		}
		else {
			agent = new AgentImpl(process, name, type);
			try {
				((ProcessImpl)process).addAgent(agent);
				agent.addAgentListener((ProcessImpl)process);
			}
			catch(ClassCastException e) {
				/* Cannot set relationship unless process is a ProcessImpl */
			}
		}
		return agent;
	}
}

