/**********************************************************************
 * Copyright (c) 2003, 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: XMLtypedEventLoader.java,v 1.3 2006-02-08 19:59:50 slavescu Exp $
 *
 * Contributors:
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.hyades.loaders.common;

import org.eclipse.hyades.loaders.util.HierarchyContext;
import org.eclipse.hyades.loaders.util.LoadersUtils;
import org.eclipse.hyades.models.common.testprofile.TPFExecutionEvent;
import org.eclipse.hyades.models.common.testprofile.TPFExecutionType;
import org.eclipse.hyades.models.common.testprofile.TPFTypedEvent;
import org.eclipse.hyades.models.common.testprofile.impl.Common_TestprofileFactoryImpl;

/**
 * @author amathur
 *
 * This is the typed event loader
 */
public class XMLtypedEventLoader extends XMLexecutionEventLoader {
	// static variables
	protected final static String EVENT_TYPED_TYPE = "type";
	
	// Attribute variables
	protected TPFExecutionType type = null;

	/* (non-Javadoc)
	 * @see org.eclipse.hyades.loaders.util.XMLFragmentLoader#initialize(org.eclipse.hyades.loaders.util.HierarchyContext, java.lang.String)
	 */
	public void initialize(HierarchyContext context, String name) {
		super.initialize(context, name);
		type = null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.hyades.loaders.util.XMLFragmentLoader#addAttribute(java.lang.String, java.lang.String)
	 */
	public void addAttribute(String name, String value) {
		printAttribute(name, value);
		switch (LoadersUtils.getHashCode(name)) {
		case ExecutionConstants.EVENT_TYPED_TYPE_int :
			type = TPFExecutionType.get(Integer.parseInt(value));
			break;

		default:
			super.addAttribute(name, value);
			break;
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.hyades.loaders.util.XMLFragmentLoader#addYourselfInContext()
	 */
	public void addYourselfInContext() {
		
		// Create the event
		TPFTypedEvent typedEvent = Common_TestprofileFactoryImpl.eINSTANCE.createTPFTypedEvent();
		
		// Now populate the specialized attributes
		typedEvent.setType(type);
		
		// populate the event with the base event attributes
		event = (TPFExecutionEvent) typedEvent;
		super.addYourselfInContext();
		super.addYourselfToContainer();
	}
	
	/**
	 * @see org.eclipse.hyades.loaders.common.XMLexecutionEventLoader#isIdentical(org.eclipse.hyades.models.common.testprofile.TPFExecutionEvent)
	 */
	protected boolean isIdentical(TPFExecutionEvent match) {
		// Make basic comparison
		if (!super.isIdentical(match)) {
			return false;
		}
		
		if (!((TPFTypedEvent)match).getType().equals(((TPFTypedEvent) event).getType())) {
			return false;
		}
		
		// Everything matched
		return true;
	}
	
	/**
	 * @see org.eclipse.hyades.loaders.common.XMLexecutionEventLoader#overwrite(org.eclipse.hyades.models.common.testprofile.TPFExecutionEvent)
	 */
	protected void overwrite(TPFExecutionEvent match) {
		super.overwrite(match);
		
		// Nothing extra here since we verified the type is the same!
	}
	
}
