/*******************************************************************************
 * Copyright (c) 2005, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: TestCasesForm.java,v 1.7 2008-01-21 17:19:42 paules Exp $
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.test.ui.editor.form;

import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.hyades.models.common.testprofile.Common_TestprofilePackage;
import org.eclipse.hyades.models.common.testprofile.TPFTestSuite;
import org.eclipse.hyades.test.ui.UiPlugin;
import org.eclipse.hyades.test.ui.editor.extension.BaseEditorExtension;
import org.eclipse.hyades.test.ui.editor.extension.TestSuiteEditorExtension;
import org.eclipse.hyades.test.ui.editor.form.base.IHasEStructuralFeature;
import org.eclipse.hyades.test.ui.editor.form.util.EditorForm;
import org.eclipse.hyades.test.ui.editor.form.util.NamedElementSection;
import org.eclipse.hyades.test.ui.editor.form.util.WidgetFactory;
import org.eclipse.hyades.test.ui.internal.action.AddTestCaseAction;
import org.eclipse.hyades.test.ui.internal.editor.form.util.EObjectTreeSection;
import org.eclipse.hyades.test.ui.internal.resources.UiPluginResourceBundle;
import org.eclipse.hyades.test.ui.internal.util.ContextIds;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

/**
 * @deprecated 
 * <p>The implementation of this class is not based on Eclipse Forms(a.k.a. "flat" look) 
 * framework that was made available publically in Eclipse 3.0. If you would like your editor(s) to be 
 * built with the Eclipse Forms technology, this is NOT the class to extend. To know more about 
 * Eclipse Forms: <br>
 * <a href="http://dev.eclipse.org/viewcvs/index.cgi/%7Echeckout%7E/pde-ui-home/working/EclipseForms/EclipseForms.html">Eclipse Forms Programming Guide</a>
 * </p>
 * 
 * @author marcelop, bcormier
 * @since 3.0.0
 */
public class TestCasesForm
extends EditorForm implements ISelectionChangedListener, IHasEStructuralFeature
{
	private EObjectTreeSection testCasesSection;
	private NamedElementSection namedElementSection;
	private AddTestCaseAction addTestCaseAction;
	private boolean firstActivation = true;

	/**
	 * Constructor for the test cases form.
	 * @param testSuiteEditorExtension the parent test context editor
	 * @param widgetFactory	the parent widget factory
	 */
	public TestCasesForm(BaseEditorExtension testSuiteEditorExtension, WidgetFactory widgetFactory)
	{
		super(testSuiteEditorExtension, widgetFactory);
		setHeadingText(UiPluginResourceBundle.W_TST_CASES); 
	}

	/**
	 * Dispose the widgets.
	 */
	public void dispose()
	{
		testCasesSection.getTreeViewer().removeSelectionChangedListener(this);
		testCasesSection.dispose();
		namedElementSection.dispose();
		addTestCaseAction.dispose();

		super.dispose();
	}

	/**
	 * Returns the test context manipulated by this form.
	 * @return TPFTestSuite the test context
	 */
	protected TPFTestSuite getTestSuite()
	{
		return ((TestSuiteEditorExtension)getBaseEditorExtension()).getTestSuite();
	}

	/**
	 * Creates the control and adds a help handler to the control.
	 * @return Control the created control
	 * @see org.eclipse.hyades.test.ui.editor.form.util.EditorForm#createControl()
	 */
	public Control createControl()
	{
		Control control = super.createControl();
		UiPlugin.getDefault().getWorkbench().getHelpSystem().setHelp(getControl(), UiPlugin.getID() + ContextIds.TESTCASE_FORM);
		return control;
	}

	/**
	 * Creates the contents of this editor form.
	 * @param parent the parent composite
	 * @see org.eclipse.hyades.test.ui.editor.form.util.EditorForm#createEditorFormContents(org.eclipse.swt.widgets.Composite)
	 */
	protected void createEditorFormContents(Composite parent)
	{
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		gridLayout.makeColumnsEqualWidth = true;
		gridLayout.marginWidth = 10;
		gridLayout.horizontalSpacing = 15;
		parent.setLayout(gridLayout);

		Composite leftColumn = createColumn(parent);
		Composite rightColumn = createColumn(parent);

		addTestCaseAction = new AddTestCaseAction();
		testCasesSection = new EObjectTreeSection(this, Common_TestprofilePackage.eINSTANCE.getTPFTestSuite_TestCases(), addTestCaseAction);
		registerSection(testCasesSection);
		testCasesSection.setHeaderText(UiPluginResourceBundle.W_TST_CASES); 
		testCasesSection.setCollapsable(true);
		Control control = testCasesSection.createControl(leftColumn, getWidgetFactory());
		control.setLayoutData(new GridData(GridData.FILL_BOTH | GridData.VERTICAL_ALIGN_BEGINNING));
		UiPlugin.getDefault().getWorkbench().getHelpSystem().setHelp(control, UiPlugin.getID() + ContextIds.TESTCASE_SECT_FORM);
		UiPlugin.getDefault().getWorkbench().getHelpSystem().setHelp(testCasesSection.getTreeViewer().getControl(), UiPlugin.getID() + ContextIds.TESTCASE_TREE_FORM);

		namedElementSection = new NamedElementSection(this);
		namedElementSection.setHeaderText(UiPluginResourceBundle.EDT_GENERAL_INFO); 
		registerSection(namedElementSection);
		control = namedElementSection.createControl(rightColumn, getWidgetFactory());
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL | GridData.VERTICAL_ALIGN_BEGINNING);
		control.setLayoutData(gridData);
		UiPlugin.getDefault().getWorkbench().getHelpSystem().setHelp(control, UiPlugin.getID() + ContextIds.TESTCASE_NAME_FORM);
	}

	/**
	 * Used to load the persisted values in the controls in this method.  
	 * @see org.eclipse.hyades.test.ui.editor.form.util.EditorForm#load()
	 */
	public void load()
	{
		addTestCaseAction.setTestSuite(getTestSuite());

		testCasesSection.getTreeViewer().removeSelectionChangedListener(this);
		testCasesSection.setInput(getTestSuite());
		testCasesSection.getTreeViewer().addSelectionChangedListener(this);
	}

	/**
	 * Reveals and selects the given element within this target view.
	 * @param selection the new element to select
	 * @see org.eclipse.ui.part.ISetSelectionTarget#selectReveal(org.eclipse.jface.viewers.ISelection)
	 */
	public void selectReveal(ISelection selection)
	{
		testCasesSection.selectReveal(selection);
	}

	/**
	 * Notifies that the selection has changed.
	 * @param event event object describing the change
	 * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
	 */
	public void selectionChanged(SelectionChangedEvent event)
	{
		if(event.getSource() == testCasesSection.getTreeViewer())
		{
			IStructuredSelection structuredSelection = (IStructuredSelection)getSelection();
			namedElementSection.setInput(structuredSelection);
		}
	}

	/**
	 * Returns the current selection for this editor extension.
	 * @return the current selection
	 */
	public ISelection getSelection()
	{
		return testCasesSection.getTreeViewer().getSelection();
	}

	/**
	 * Returns whether this form has been activated yet.
	 * @return the activation state
	 */
	public boolean activated()
	{
		if(firstActivation)
		{
			firstActivation = false;
			testCasesSection.setFocus();
		}

		return super.activated();
	}

	/**
	 * Returns the structural feature of this test case form. i.e. the test cases structural feature.
	 * @return the structural feature that corresponds to test cases
	 * @see org.eclipse.hyades.test.ui.editor.form.IHasEStructuralFeature#getStructuralFeature()
	 */
	public EStructuralFeature getStructuralFeature() {
		return Common_TestprofilePackage.eINSTANCE.getTPFTestSuite_TestCases();
	}

}
